#pragma once

//Copyright (C) cocosimulator.org 2014
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

#include "Water_DLL.h"

// base class for all CAPE-OPEN objects
// implements identification interface and error interfaces

// before returning a CAPE-OPEN error code from any
// CAPE-OPEN function, call SetError

// Example:
//  SetError(L"Could not calculate: division by zero",L"Calculate",L"ICapeUnit");
//  return ECapeComputationHr;
// when no clear error code, return ECapeUnknownHR

class CAPEOPENBASE :
	public COMObject,
	public ICapeIdentification,
	public ECapeRoot,
	public ECapeUser,
	//only the corresponding error codes can be returned:
	public ECapeUnknown,
	public ECapeInvalidArgument
	{
private:

	// members
	wstring name,desc,errDesc,errScope,errIface;
	bool cannotChangeName;

public:

	// constructors/destructor
	CAPEOPENBASE(const OLECHAR *initialName=nullptr,const OLECHAR *initialDescription=nullptr,bool fixedName=false) {
		//default object name
		if (initialName) {
			name=initialName;
		} else {
			name=L"<unnamed>";
		}
		if (initialDescription) {
			desc=initialDescription;
		}
		cannotChangeName=fixedName;
	}

	//call before any CAPE-OPEN error return
	void SetError(const OLECHAR *desc,const OLECHAR *scope,const OLECHAR *iface) {
		errDesc=desc;
		errScope=scope;
		errIface=iface;
	}

	IMPLEMENT_IDISPATCH

	void *InternalQueryInterface(REFIID riid) override {
		if (riid==IID_ICapeIdentification) {
			return (ICapeIdentification*)this;
		}
		if (riid==IID_IDispatch) {
			return (ICapeIdentification*)this;
		}
		if (riid==IID_ECapeRoot) {
			return (ECapeRoot*)this;
		}
		if (riid==IID_ECapeUser) {
			return (ECapeUser*)this;
		}
		if (riid==IID_ECapeUnknown) {
			return (ECapeUnknown*)this;
		}
		if (riid==IID_ECapeInvalidArgument) {
			return (ECapeInvalidArgument*)this;
		}
		return nullptr;
	}

	//ICapeIdentification

	STDMETHOD(get_ComponentName)(/* [retval][out] */ BSTR *name) {
		if (!name) {
			return E_POINTER;
		}
		*name=SysAllocString(this->name.c_str());
		return NO_ERROR;
	}

	STDMETHOD(put_ComponentName)(/* [in] */ BSTR name) {
		if (cannotChangeName) {
			//this object cannot change name
			SetError(L"The name of this object cannot be changed",L"put_ComponentName",L"ICapeIdentification");
			return ECapeUnknownHR;
		}
		if (!name) {
emptyName:
			SetError(L"Name cannot be empty",L"put_ComponentName",L"ICapeIdentification");
			return ECapeUnknownHR;
		}
		if (!*name) {
			goto emptyName;
		}
		this->name=name;
		return NO_ERROR;
	}

	STDMETHOD(get_ComponentDescription)(/* [retval][out] */ BSTR *desc) {
		if (!desc) {
			return E_POINTER;
		}
		*desc=SysAllocString(this->desc.c_str());
		return NO_ERROR;
	}

	STDMETHOD(put_ComponentDescription)( /* [in] */ BSTR desc) {
		if (cannotChangeName) {
			//this object cannot change name or description
			SetError(L"The description of this object cannot be changed",L"put_ComponentDescription",L"ICapeIdentification");
			return ECapeUnknownHR;
		}
		if (!desc) {
			this->desc.clear();
		} else {
			this->desc=desc;
		}
		return NO_ERROR;
	}

	// ECapeRoot Methods

	STDMETHOD(get_name)(BSTR * name) {
		//return error name
		if (!name) {
			return E_POINTER;
		}
		*name=SysAllocString(errDesc.c_str());
		return NO_ERROR;
	}

	// ECapeUser Methods

	STDMETHOD(get_code)(long * code) {
		//not supported
		if (!code) {
			return E_POINTER;
		}
		*code=0;
		return NO_ERROR;
	}

	STDMETHOD(get_description)(BSTR * description) {
		//return error description
		if (!description) {
			return E_POINTER;
		}
		*description=SysAllocString(errDesc.c_str());
		return NO_ERROR;
	}

	STDMETHOD(get_scope)(BSTR * scope) {
		//return error scope
		if (!scope) {
			return E_POINTER;
		}
		*scope=SysAllocString(errScope.c_str());
		return NO_ERROR;
	}

	STDMETHOD(get_interfaceName)(BSTR * interfaceName) {
		//return error interface
		if (!interfaceName) {
			return E_POINTER;
		}
		*interfaceName=SysAllocString(errIface.c_str());
		return NO_ERROR;
	}

	STDMETHOD(get_operation)(BSTR * operation) {
		//error operation not supported
		if (!operation) {
			return E_POINTER;
		}
		*operation=SysAllocString(L"N/A");
		return NO_ERROR;
	}

	STDMETHOD(get_moreInfo)(BSTR * moreInfo) {
		//more info, use company URL
		if (!moreInfo) {
			return E_POINTER;
		}
		*moreInfo=SysAllocString(L"For more info, please see http://www.cocosimulator.org/");
		return NO_ERROR;
	}

};

